@@ -1,17 +1,23 @@
-import json
 
+import json
 from loguru import logger
-from uuid import uuid4
+from beanie.operators import And, In
+
 from openai import AsyncOpenAI
 from openai.types.responses import ResponseFunctionToolCall, ResponseOutputMessage, ResponseOutputText
 
-from AgentService.db import Db
-from AgentService.dtypes.db import method as dmth
-from AgentService.dtypes.chat import Chat, ChatStatus
-from AgentService.dtypes.message import Message, MessageType, ToolAnswer, Tool
-from AgentService.dtypes.storage import Storage, StorageItem
-from AgentService.types.agent_tool import AgentTool
-from AgentService.types.response import AgentResponse, AgentResponseType
+from AgentService.types import (
+    Chat, ChatToolState,
+    AgentResponse, Message,
+    Storage, StorageItem,
+    Tool, ToolAnswer
+)
+
+from AgentService.enums.chat import ChatStatus, ChatToolStateStatus
+from AgentService.enums.agent import AgentResponseType
+from AgentService.enums.message import MessageType
+
+from AgentService.agent.tool import AgentTool
 
 
 class Agent:
@@ -20,7 +26,9 @@ class Agent:
     max_tokens: int = 2048
     top_p: float = 1.0
 
-    system_prompt: str = "You are a helpful assistant"
+    instructions: str = "You are a helpful assistant"
+
+    system_prompts: list[str] = []
     prompt: str = "{text}"
 
     is_one_shot: bool = False
@@ -28,32 +36,36 @@ class Agent:
     def __init__(
         self,
         openai_key: str,
-        tools: list[AgentTool] = None,
+        tools: list[AgentTool] = None
     ):
 
         self.log = logger.bind(classname=self.__class__.__name__)
-        self.db = Db()
+
         self.client = AsyncOpenAI(api_key=openai_key)
 
         self.tools = tools if tools else []
-        self.tools_schema = [tool.to_schema for tool in self.tools]
+        self.tools_schema = [tool.openai_model_dump for tool in self.tools]
+
+        self.system_prompts = list(map(lambda x: x.replace("\t", ""), self.system_prompts))
+        self.instructions = self.instructions.replace("\t", "")
 
         self.log.debug(f"Created gpt like tools -> {self.tools_schema}")
 
-    async def __system_prompt(self, context: dict) -> str:
-        return self.system_prompt.format(**context)
+    async def __system_prompt(self, i: int, context: dict) -> str:
+        return self.system_prompts[i].format(**context)
 
     async def __prompt(self, text: str, context: dict) -> str:
         return self.prompt.format(text=text, **context)
 
     async def __generate(self, chat: Chat) -> AgentResponse:
         chat.status = ChatStatus.generating
-        await self.db.ex(dmth.UpdateOne(Chat, chat, to_update="status"))
+        await chat.save()
+
+        messages = await Message.find_many(Message.chat_id == chat.id).to_list()
 
-        messages: list[Message] = await self.db.ex(dmth.GetMany(Message, chat_id=chat.id))
         gpt_messages = []
         for message in messages:
-            data = message.gpt_dump
+            data = message.openai_model_dump
 
             if isinstance(data, list):
                 for row in data:
@@ -64,9 +76,9 @@ class Agent:
 
         tools_schema = self.tools_schema[:]
 
-        storage_item: StorageItem = await self.db.ex(dmth.GetOne(StorageItem))
+        storage_item = await StorageItem.find_one()
         if storage_item:
-            storages: list[Storage] = await self.db.ex(dmth.GetMany(Storage))
+            storages = await Storage.find_many().to_list()
 
             tools_schema.append(
                 {
@@ -79,7 +91,9 @@ class Agent:
             response = await self.client.responses.create(
                 model=self.model,
                 input=gpt_messages,
+                instructions=self.instructions,
                 temperature=self.temperature,
+                max_output_tokens=self.max_tokens,
                 top_p=self.top_p,
                 tools=tools_schema,
                 tool_choice="auto"
@@ -104,16 +118,31 @@ class Agent:
             raise TypeError(f"Got many response types in 1 output -> {responses}")
 
         if isinstance(responses[0], ResponseFunctionToolCall):
+            tools = [
+                Tool(
+                    tool_id=i.call_id,
+                    name=i.name,
+                    arguments=i.arguments
+                )
+                for i in responses
+            ]
+
+            states = [
+                ChatToolState(
+                    chat_id=chat.id,
+                    status=ChatToolStateStatus.in_progress,
+                    tool_id=tool.tool_id,
+                    name=tool.name,
+                    arguments=tool.arguments
+                )
+                for tool in tools
+            ]
+
+            await ChatToolState.insert_many(states)
+
             return AgentResponse(
                 type=AgentResponseType.tools,
-                content=[
-                    Tool(
-                        id=i.call_id,
-                        name=i.name,
-                        arguments=i.arguments
-                    )
-                    for i in responses
-                ]
+                content=tools
             )
 
         elif isinstance(responses[0], ResponseOutputMessage):
@@ -148,38 +177,45 @@ class Agent:
             )
 
     async def get_chat(self, chat_id: str, context: dict = {}) -> Chat:
-        chat: Chat = await self.db.ex(dmth.GetOne(
-            Chat,
-            chat_id=chat_id,
-            status={
-                "$in": [ChatStatus.created.value, ChatStatus.idle.value, ChatStatus.tools.value]
-            }
-        ))
+        chat = await Chat.find_one(
+            And(
+                Chat.chat_id == chat_id,
+                In(Chat.status, [ChatStatus.created, ChatStatus.idle, ChatStatus.tools])
+            )
+        )
+
         if not chat:
             chat = Chat(
-                id=uuid4().hex,
                 chat_id=chat_id,
                 status=ChatStatus.created,
                 data=context
             )
-            await self.db.ex(dmth.AddOne(Chat, chat))
+            await chat.save()
 
-        system_message: Message = await self.db.ex(dmth.GetOne(Message, chat_id=chat.id, type=MessageType.system.value))
-        if not system_message:
-            system_message = Message(
-                id=uuid4().hex,
-                chat_id=chat.id,
-                text=await self.__system_prompt(context),
-                type=MessageType.system
+        system_message = await Message.find_one(
+            And(
+                Message.chat_id == chat.id,
+                Message.type == MessageType.system
             )
-            await self.db.ex(dmth.AddOne(Message, system_message))
+        )
+        if not system_message:
+            messages = []
+
+            for i in range(len(self.system_prompts)):
+                system_message = Message(
+                    chat_id=chat.id,
+                    text=await self.__system_prompt(i, context),
+                    type=MessageType.system
+                )
+                messages.append(system_message)
+
+            await Message.insert_many(messages)
 
         return chat
 
     async def proccess_answer(self, answer: AgentResponse, chat: Chat) -> AgentResponse:
         if answer.type == AgentResponseType.tools:
             bot_message = Message(
-                id=uuid4().hex,
                 chat_id=chat.id,
                 type=MessageType.tools,
                 tools=answer.content
@@ -188,7 +224,6 @@ class Agent:
 
         elif answer.type == AgentResponseType.answer:
             bot_message = Message(
-                id=uuid4().hex,
                 chat_id=chat.id,
                 type=MessageType.assistant,
                 text=answer.content
@@ -198,41 +233,53 @@ class Agent:
         else:
             raise ValueError(f"wrong answer type got {answer.type}, expected {AgentResponseType} like")
 
-        await self.db.ex(dmth.AddOne(Message, bot_message))
-        await self.db.ex(dmth.UpdateOne(Chat, chat, to_update="status"))
+        await bot_message.insert()
+        await chat.save()
 
         return answer
 
-    async def skip_tools(self, chat) -> str:
-        messages: list[Message] = await self.db.ex(dmth.GetMany(Message, chat_id=chat.id))
+    async def skip_tools(self, chat: Chat) -> str:
+        messages = await Message.find_many(Message.chat_id == chat.id).to_list()
         last_message: Message = messages[-1]
 
         if not last_message.tools:
             chat.status = ChatStatus.idle
-            await self.db.ex(dmth.UpdateOne(Chat, chat, to_update="status"))
+            await chat.save()
+
             return chat
 
-        new_messages = [
-            Message(
-                id=uuid4().hex,
-                chat_id=chat.id,
-                type=MessageType.tool_answer,
-                tool_answer=ToolAnswer(
-                    id=tool.id,
-                    name=tool.name,
-                    text="Tool call skipped."
+        new_messages = []
+        for tool in last_message.tools:
+            tool_state = await ChatToolState.find_one(
+                And(
+                    ChatToolState.chat_id == chat.id,
+                    ChatToolState.tool_id == tool.tool_id,
                 )
             )
-            for tool in last_message.tools
-        ] + [
+            if tool_state:
+                tool_state.status = ChatToolStateStatus.completed
+                await tool_state.save()
+
+            new_messages.append(
+                Message(
+                    chat_id=chat.id,
+                    type=MessageType.tool_answer,
+                    tool_answer=ToolAnswer(
+                        tool_id=tool.tool_id,
+                        name=tool.name,
+                        text="Tool call skipped."
+                    )
+                )
+            )
+
+        new_messages.append(
             Message(
-                id=uuid4().hex,
                 chat_id=chat.id,
                 type=MessageType.assistant,
                 text="skip"
             )
-        ]
-        await self.db.ex(dmth.AddMany(Message, new_messages))
+        )
+        await Message.insert_many(new_messages)
 
         return chat
 
@@ -242,12 +289,11 @@ class Agent:
             chat = await self.skip_tools(chat)
 
         user_message = Message(
-            id=uuid4().hex,
             chat_id=chat.id,
             text=await self.__prompt(text, context),
             type=MessageType.user
         )
-        await self.db.ex(dmth.AddOne(Message, user_message))
+        await user_message.insert()
 
         answer = await self.generate(chat=chat)
         await self.proccess_answer(answer, chat)
@@ -262,13 +308,23 @@ class Agent:
 
         new_messages = []
         for tool_answer in tool_answers:
+            tool_state = await ChatToolState.find_one(
+                And(
+                    ChatToolState.chat_id == chat.id,
+                    ChatToolState.tool_id == tool_answer.tool_id,
+                )
+            )
+            if tool_state:
+                tool_state.status = ChatToolStateStatus.completed
+                await tool_state.save()
+
             new_messages.append(Message(
-                id=uuid4().hex,
                 chat_id=chat.id,
                 type=MessageType.tool_answer,
                 tool_answer=tool_answer
             ))
-        await self.db.ex(dmth.AddMany(Message, new_messages))
+
+        await Message.insert_many(new_messages)
 
         answer = await self.generate(chat=chat)
         await self.proccess_answer(answer, chat)
@@ -294,7 +350,7 @@ class Agent:
         self.log.info(f"Created storage -> {vector_store.id}")
 
         return Storage(
-            id=vector_store.id,
+            vector_store_id=vector_store.id,
             key=key
         )
 
