@@ -21,7 +21,7 @@ async def lifespan(app: fastapi.FastAPI):
     return
 
 
-app = fastapi.FastAPI()
+app = fastapi.FastAPI(lifespan=lifespan)
 app.include_router(chat_router)
 app.include_router(storage_router)
 
