@@ -1,5 +1,5 @@
 
-from beanie import Document
+from . import Document
 from beanie.odm.documents import PydanticObjectId
 
 from .meta import TimestampMixin
