@@ -1,5 +1,5 @@
 
-from beanie import Document
+from . import Document
 
 from .meta import TimestampMixin
 
